# coding:utf-8

from urllib.parse import urljoin

__project__ = "rfc-kits"
__version__ = "0.1.2"
__description__ = "Search, View and Download RFCs."
__urlhome__ = "https://github.com/bondbox/rfc-kits"
__urlcode__ = __urlhome__
__urldocs__ = __urlhome__
__urlbugs__ = urljoin(__urlhome__, "issues")

# author
__author__ = "Mingzhe Zou"
__author_email__ = "zoumingzhe@outlook.com"
